unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls;

type

  TColor4 = packed array[0..3] of byte;
  PColor4 = ^TColor4;

  TColor3 = packed array[0..2] of byte;
  PColor3 = ^TColor3;

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    Button7: TButton;
    Button8: TButton;
    Image1: TImage;
    Memo1: TMemo;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure Button7Click(Sender: TObject);
    procedure Button8Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    TestColor4: TColor4;
    TestColor3: TColor3;
    TestColor: TColor;

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

uses
  GraphType, LCLType, IntfGraphics, FPImage,
  BGRABitmap, BGRABitmapTypes;

const
  IMG_WIDTH = 3000;
  IMG_HEIGHT = 3000;

function RGB4(ARed, AGreen, ABlue: Byte): TColor4;
begin
{$IF DEFINED(LCLWin32) or DEFINED(LCLQt5) or DEFINED (LCLCocoa) }
  Result[0] := ABlue;
  Result[1] := AGreen;
  Result[2] := ARed;
  Result[3] := 255;
{$ELSE}
  Result[0] := ARed;
  Result[1] := AGreen;
  Result[2] := ABlue;
  Result[3] := 255;
{$ENDIF}
end;

function RGB3(ARed, AGreen, ABlue: Byte): TColor3;
begin
  Result[0] := ABlue;
  Result[1] := AGreen;
  Result[2] := ARed;
end;


{ TForm1 }

procedure TForm1.Button1Click(Sender: TObject);
var
  i: Integer;
  p: PBGRAPixel;
  img: TBGRABitmap;
  t: TDateTime;
begin
  Image1.Picture.Clear;
  Image1.Refresh;

  t := now;
  img := TBGRABitmap.Create(IMG_WIDTH, IMG_HEIGHT);
  p := img.data;
  for i := 0 to img.NbPixels -1  do
  begin
    p^ := TestColor;
    inc(p);
  end;
  img.InvalidateBitmap;
  t := Now-t;
  Image1.Picture.Assign(img);
  img.free;

  Caption := 'BGRABitmap Data: ' + FormatDateTime('s.zzz "s"', t);
  Memo1.Lines.Add(Caption);
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  bmp: TBitmap;
  x, y: Integer;
  t: TDateTime;
begin
  Image1.Picture.Clear;
  Image1.Refresh;

  t := Now;
  bmp := TBitmap.Create;
  bmp.SetSize(IMG_WIDTH, IMG_HEIGHT);;
  for y := 0 to bmp.Height-1 do
    for x := 0 to bmp.Width-1 do
      bmp.Canvas.Pixels[x, y] := TestColor;
  t := Now-t;
  Image1.Picture.Assign(bmp);
  bmp.Free;

  Caption := 'Bitmap Pixels: ' + FormatDateTime('s.zzz "s"', t);
  Memo1.Lines.Add(Caption);
end;

procedure TForm1.Button3Click(Sender: TObject);
var
  bmp: TBitmap;
  hBmp, hMask: THandle;
  img: TLazIntfImage;
  x, y: Integer;
  t: TDateTime;
  clr: TFPColor;
begin
  Image1.Picture.Clear;
  Image1.Refresh;

  t := Now;
  bmp := TBitmap.Create;
  bmp.SetSize(IMG_WIDTH, IMG_HEIGHT);
  clr := TColorToFPColor(TestColor);
  img := bmp.CreateIntfImage;
  for y := 0 to bmp.Height-1 do
    for x := 0 to bmp.Width-1 do
      img.Colors[x, y] := clr;
  img.CreateBitmaps(hBmp, hMask);
  bmp.Handle := hBmp;
  bmp.MaskHandle := hMask;
  t := Now-t;
  Image1.Picture.Assign(bmp);
  bmp.Free;
  Caption := 'LazIntfImage: ' + FormatDateTime('s.zzz "s"', t);
  Memo1.Lines.Add(Caption);
end;

procedure TForm1.Button4Click(Sender: TObject);
var
  x, y: Integer;
  img: TBGRABitmap;
  t: TDateTime;
begin
  Image1.Picture.Clear;
  Image1.Refresh;

  t := now;
  img := TBGRABitmap.Create(IMG_WIDTH, IMG_HEIGHT);
  for y := 0 to IMG_HEIGHT-1 do
    for x := 0 to IMG_WIDTH-1 do
      img.SetPixel(x, y, TestColor);
  img.InvalidateBitmap;
  t := Now - t;
  Image1.Picture.Assign(img);
  img.free;

  Caption := 'BGRABitmap SetPixel: ' + FormatDateTime('s.zzz "s"', t);
  Memo1.Lines.Add(Caption);
end;


procedure TForm1.Button5Click(Sender: TObject);
var
  bmp: TBitmap;
  x, y: Integer;
  t: TDateTime;
  P: Pointer;
begin
  Image1.Picture.Clear;
  Image1.Refresh;

  t := Now;
  bmp := TBitmap.Create;
  bmp.PixelFormat := pf32Bit;
  bmp.SetSize(IMG_WIDTH, IMG_HEIGHT);;
  for y := 0 to bmp.Height-1 do
  begin
    P := bmp.ScanLine[y];
    for x := 0 to bmp.Width-1 do
    begin
      PColor4(P)^ := TestColor4;
      inc(P, 4);
    end;
  end;
  t := Now-t;
  Image1.Picture.Assign(bmp);
  bmp.Free;

  Caption := 'Bitmap ScanLine (32bit): ' + FormatDateTime('s.zzz "s"', t);
  Memo1.Lines.Add(Caption);
end;

procedure TForm1.Button6Click(Sender: TObject);
var
  bmp: TBitmap;
  x, y: Integer;
  t: TDateTime;
  P: PByte;
begin
  Image1.Picture.Clear;
  Image1.Refresh;

  t := Now;
  bmp := TBitmap.Create;
  bmp.PixelFormat := pf32Bit;
  bmp.SetSize(IMG_WIDTH, IMG_HEIGHT);
  for y := 0 to bmp.Height-1 do
  begin
    P := bmp.RawImage.GetLineStart(y);
    for x := 0 to bmp.Width-1 do
    begin
      PColor4(P)^ := TestColor4;
      inc(P, 4);
    end;
  end;
  t := Now-t;
  Image1.Picture.Assign(bmp);
  bmp.Free;

  Caption := 'Bitmap RawImage (32bit): ' + FormatDateTime('s.zzz "s"', t);
  Memo1.Lines.Add(Caption);
end;

procedure TForm1.Button7Click(Sender: TObject);
var
  bmp: TBitmap;
  x, y: Integer;
  t: TDateTime;
  P: Pointer;
begin
  Image1.Picture.Clear;
  Image1.Refresh;

  t := Now;
  bmp := TBitmap.Create;
  bmp.PixelFormat := pf24Bit;
  bmp.SetSize(IMG_WIDTH, IMG_HEIGHT);
  for y := 0 to bmp.Height-1 do
  begin
    P := bmp.ScanLine[y];
    for x := 0 to bmp.Width-1 do
    begin
      PColor3(P)^ := TestColor3;
      {$IFDEF WINDOWS}
      inc(P, 3);
      {$ELSE}
      inc(P, 4);
      {$ENDIF}
    end;
  end;
  t := Now-t;
  Image1.Picture.Assign(bmp);
  bmp.Free;

  Caption := 'Bitmap ScanLine (24bit): ' + FormatDateTime('s.zzz "s"', t);
  Memo1.Lines.Add(Caption);
end;

procedure TForm1.Button8Click(Sender: TObject);
var
  bmp: TBitmap;
  img: TRawImage;
  x, y: Integer;
  t: TDateTime;
  P: PByte;
begin
  Image1.Picture.Clear;
  Image1.Refresh;

  t := Now;
  bmp := TBitmap.Create;
  bmp.PixelFormat := pf24Bit;
  bmp.SetSize(IMG_WIDTH, IMG_HEIGHT);
  for y := 0 to bmp.Height-1 do
  begin
    P := bmp.RawImage.GetLineStart(y);
    for x := 0 to bmp.Width-1 do
    begin
      PColor3(P)^ := TestColor3;
      {$IFDEF WINDOWS}
      inc(P, 3);
      {$ELSE}
      inc(P, 4);
      {$ENDIF}
    end;
  end;
  t := Now-t;
  Image1.Picture.Assign(bmp);
  bmp.Free;

  Caption := 'Bitmap RawImage (24bit): ' + FormatDateTime('s.zzz "s"', t);
  Memo1.Lines.Add(Caption);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  TestColor := clBlue;
  TestColor4 := RGB4(0, 0, 255);
  TestColor3 := RGB3(0, 0, 255);

  // No 24-bit bitmaps on cocoa.
  {$IF DEFINED(LCLCocoa)}
  Button7.Hide;
  Button8.Hide;
  {$ENDIF}

end;

end.

